# These are GBIF api enums, rather than Julia enums.
# We also include the doc string from the gbif online docs
# so we can include it in docs here.
const ENUMS = (
    basisOfRecord = Dict(
        :FOSSIL_SPECIMEN => "An occurrence record describing a fossilized specimen.",
        :HUMAN_OBSERVATION => "An occurrence record describing an observation made by one or more people.",
        :LITERATURE => "Deprecated. use BasisOfRecord.OCCURRENCE instead.",
        :LIVING_SPECIMEN => "An occurrence record describing a living specimen, e.g.",
        :MACHINE_OBSERVATION => "An occurrence record describing an observation made by a machine.",
        :MATERIAL_CITATION => "A reference to or citation of one, a part of, or multiple specimens in scholarly publications.",
        :MATERIAL_SAMPLE => "An occurrence record based on samples taken from other specimens or the environment.",
        :OBSERVATION => "An occurrence record describing an observation.",
        :OCCURRENCE => "An existence of an Organism (sensu http://rs.tdwg.org/dwc/terms/Organism) at a particular place at a particular time.",
        :PRESERVED_SPECIMEN => "An occurrence record describing a preserved specimen.",
        :UNKNOWN => "Deprecated. replaced by BasisOfRecord.OCCURRENCE.",
    ),
    continent = Dict(
        :AFRICA => "Africa.",
        :ANTARCTICA => "Antarctica.",
        :ASIA => "Asia.",
        :EUROPE => "Europe.",
        :NORTH_AMERICA => "North America",
        :OCEANIA => "Oceania.",
        :SOUTH_AMERICA => "South America.",
    ),
    establishmentMeans = Dict(
        :INTRODUCED => "Deprecated.",
        :INVASIVE => "Deprecated.",
        :MANAGED => "Deprecated.",
        :NATIVE => "Deprecated.",
        :NATURALISED => "Deprecated.",
        :UNCERTAIN => "Deprecated.",
    ),
    habitat = Dict(
        :FRESHWATER => "Freshwater habitats including rivers, lakes, ponds, wetlands, bogs, marsh, swamp and brackish waters!",
        :MARINE => "Marine habitats including the deep & open ocean, reefs and estuaries.",
        :TERRESTRIAL => "Terrestrial habitats cover all habitats on land including forests, deserts, grasslands, meadows, tundra, mangroves, farmland.",
    ),
    issue = Dict(
        :ACCEPTED_NAME_MISSING => "Synonym lacking an accepted name.",
        :ACCEPTED_NAME_NOT_UNIQUE => "Synonym has a verbatim accepted name which is not unique and refers to several records.",
        :ACCEPTED_NAME_USAGE_ID_INVALID => "The value for dwc:acceptedNameUsageID could not be resolved.",
        :ALT_IDENTIFIER_INVALID => "At least one alternative identifier extension record attached to this name usage is invalid.",
        :BACKBONE_MATCH_AGGREGATE => "Name usage could only be matched to a GBIF backbone species, but was in fact a broader species aggregate/complex.",
        :BACKBONE_MATCH_FUZZY => "Deprecated. because there should be no fuzzy matching being used anymore for matching checklist names",
        :BACKBONE_MATCH_NONE => "Name usage could not be matched to the GBIF backbone.",
        :BASIONYM_AUTHOR_MISMATCH => "The authorship of the original name does not match the authorship in brackets of the actual name.",
        :BIB_REFERENCE_INVALID => "At least one bibliographic reference extension record attached to this name usage is invalid.",
        :CHAINED_SYNOYM => "If a synonym points to another synonym as its accepted taxon the chain is resolved.",
        :CLASSIFICATION_NOT_APPLIED => "The denormalized classification could not be applied to the name usage.",
        :CLASSIFICATION_RANK_ORDER_INVALID => "The given ranks of the names in the classification hierarchy do not follow the hierarchy of ranks.",
        :CONFLICTING_BASIONYM_COMBINATION => "There have been more than one accepted name in a homotypical basionym group of names.",
        :DESCRIPTION_INVALID => "At least one description extension record attached to this name usage is invalid.",
        :DISTRIBUTION_INVALID => "At least one distribution extension record attached to this name usage is invalid.",
        :HOMONYM => "A not synonymized homonym exists for this name in some other backbone source which have been ignored at build time.",
        :MULTIMEDIA_INVALID => "At least one multimedia extension record attached to this name usage is invalid.",
        :NAME_PARENT_MISMATCH => "The (accepted) bi/trinomial name does not match the parent name and should be recombined into the parent genus/species.",
        :NO_SPECIES => "The group (currently only genera are tested) are lacking any accepted species GBIF backbone specific issue.",
        :NOMENCLATURAL_STATUS_INVALID => "dwc:nomenclaturalStatus could not be interpreted",
        :ORIGINAL_NAME_DERIVED => "Record has a original name (basionym) relationship which was derived from name & authorship comparison, but did not exist explicitly in the data.",
        :ORIGINAL_NAME_NOT_UNIQUE => "Record has a verbatim original name (basionym) which is not unique and refers to several records.",
        :ORIGINAL_NAME_USAGE_ID_INVALID => "The value for dwc:originalNameUsageID could not be resolved.",
        :ORTHOGRAPHIC_VARIANT => "A potential orthographic variant exists in the backbone.",
        :PARENT_CYCLE => "The child parent classification resulted into a cycle that needed to be resolved/cut.",
        :PARENT_NAME_NOT_UNIQUE => "Record has a verbatim parent name which is not unique and refers to several records.",
        :PARENT_NAME_USAGE_ID_INVALID => "The value for dwc:parentNameUsageID could not be resolved.",
        :PARTIALLY_PARSABLE => "The beginning of the scientific name string was parsed, but there is additional information in the string that was not understood.",
        :PUBLISHED_BEFORE_GENUS => "A bi/trinomial name published earlier than the parent genus was published.",
        :RANK_INVALID => "dwc:taxonRank could not be interpreted",
        :RELATIONSHIP_MISSING => "There were problems representing all name usage relationships, i.e.",
        :SCIENTIFIC_NAME_ASSEMBLED => "The scientific name was assembled from the individual name parts and not given as a whole string.",
        :SPECIES_PROFILE_INVALID => "At least one species profile extension record attached to this name usage is invalid.",
        :TAXONOMIC_STATUS_INVALID => "dwc:taxonomicStatus could not be interpreted",
        :TAXONOMIC_STATUS_MISMATCH => "",
        :UNPARSABLE => "The scientific name string could not be parsed at all, but appears to be a parsable name type, i.e.",
        :VERNACULAR_NAME_INVALID => "At least one vernacular name extension record attached to this name usage is invalid.",
    ),
    nameType = Dict(
        :BLACKLISTED => "A name that has been flagged by the name parser due to the use of blacklisted epithets/words.",
        :CANDIDATUS => "Candidatus is a component of the taxonomic name for a bacterium that cannot be maintained in a Bacteriology Culture Collection.",
        :CULTIVAR => "A cultivated plant name.",
        :DOUBTFUL => "Doubtful whether this is a scientific name at all.",
        :HYBRID => "A hybrid formula (not a hybrid name).",
        :INFORMAL => "A scientific name with some informal addition like \"cf.\" or indetermined like Abies spec.",
        :NO_NAME => "Surely not a scientific name of any kind.",
        :OTU => "Operational Taxonomic Unit.",
        :PLACEHOLDER => "A placeholder name like \"incertae sedis\" or \"unknown genus\".",
        :SCIENTIFIC => "A scientific latin name that might contain authorship but is not any of the other name types below (virus, hybrid, cultivar, etc).",
        :VIRUS => "",
    ),
    rank = Dict(
        :ABERRATION => "Zoological legacy rank",
        :BIOVAR => "Microbial rank based on biochemical or physiological properties.",
        :CHEMOFORM => "Microbial infrasubspecific rank based on chemical constitution.",
        :CHEMOVAR => "Microbial rank based on production or amount of production of a particular chemical.",
        :CLASS => "",
        :COHORT => "Sometimes used in zoology, e.g.",
        :CONVARIETY => "A group of cultivars.",
        :CULTIVAR => "",
        :CULTIVAR_GROUP => "Rank in use from the code for cultivated plants.",
        :DOMAIN => "",
        :FAMILY => "",
        :FORM => "",
        :FORMA_SPECIALIS => "Microbial infrasubspecific rank.",
        :GENUS => "",
        :GRANDORDER => "",
        :GREX => "The term grex has been coined to expand botanical nomenclature to describe hybrids of orchids.",
        :INFRACLASS => "",
        :INFRACOHORT => "",
        :INFRAFAMILY => "",
        :INFRAGENERIC_NAME => "used for any other unspecific rank below genera and above species.",
        :INFRAGENUS => "",
        :INFRAKINGDOM => "",
        :INFRALEGION => "",
        :INFRAORDER => "",
        :INFRAPHYLUM => "",
        :INFRASPECIFIC_NAME => "used for any other unspecific rank below species.",
        :INFRASUBSPECIFIC_NAME => "used also for any other unspecific rank below subspecies.",
        :INFRATRIBE => "",
        :KINGDOM => "",
        :LEGION => "Sometimes used in zoology, e.g.",
        :MAGNORDER => "",
        :MORPH => "Zoological legacy rank",
        :MORPHOVAR => "Microbial rank based on morphological characterislics.",
        :NATIO => "Zoological legacy rank",
        :ORDER => "",
        :OTHER => "Any other rank we cannot map to this enumeration",
        :PARVCLASS => "",
        :PARVORDER => "",
        :PATHOVAR => "Microbial rank based on pathogenic reactions in one or more hosts.",
        :PHAGOVAR => "Microbial infrasubspecific rank based on reactions to bacteriophage.",
        :PHYLUM => "",
        :PROLES => "Botanical legacy rank",
        :RACE => "Botanical legacy rank",
        :SECTION => "",
        :SERIES => "",
        :SEROVAR => "Microbial infrasubspecific rank based on antigenic characteristics.",
        :SPECIES => "",
        :SPECIES_AGGREGATE => "A loosely defined group of species.",
        :STRAIN => "A microbial strain.",
        :SUBCLASS => "",
        :SUBCOHORT => "",
        :SUBFAMILY => "",
        :SUBFORM => "",
        :SUBGENUS => "",
        :SUBKINGDOM => "",
        :SUBLEGION => "",
        :SUBORDER => "",
        :SUBPHYLUM => "",
        :SUBSECTION => "",
        :SUBSERIES => "",
        :SUBSPECIES => "",
        :SUBTRIBE => "",
        :SUBVARIETY => "",
        :SUPERCLASS => "",
        :SUPERCOHORT => "",
        :SUPERFAMILY => "",
        :SUPERKINGDOM => "",
        :SUPERLEGION => "",
        :SUPERORDER => "",
        :SUPERPHYLUM => "",
        :SUPERTRIBE => "",
        :SUPRAGENERIC_NAME => "Used for any other unspecific rank above genera.",
        :TRIBE => "",
        :UNRANKED => "",
        :VARIETY => "",
   ),
   threat = Dict(
        :CRITICALLY_ENDANGERED => "A taxon is Critically Endangered when the best available evidence indicates that it meets any of the criteria A to E for Critically Endangered (see Section V), and it is therefore considered to be facing an extremely high risk of extinction in the wild.",
        :DATA_DEFICIENT => "A taxon is Data Deficient when there is inadequate information to make a direct, or indirect, assessment of its risk of extinction based on its distribution and/or population status.",
        :ENDANGERED => "A taxon is Endangered when the best available evidence indicates that it meets any of the criteria A to E for Endangered (see Section V), and it is therefore considered to be facing a very high risk of extinction in the wild.",
        :EXTINCT => "A taxon is Extinct when there is no reasonable doubt that the last individual has died.",
        :EXTINCT_IN_THE_WILD => "A taxon is Extinct in the Wild when it is known only to survive in cultivation, in captivity or as a naturalized population (or populations) well outside the past range.",
        :LEAST_CONCERN => "A taxon is Least Concern when it has been evaluated against the criteria and does not qualify for Critically Endangered, Endangered, Vulnerable or Near Threatened.",
        :NEAR_THREATENED => "A taxon is Near Threatened when it has been evaluated against the criteria but does not qualify for Critically Endangered, Endangered or Vulnerable now, but is close to qualifying for or is likely to qualify for a threatened category in the near future.",
        :NOT_APPLICABLE => "Category for a taxon deemed to be ineligible for assessment at a regional level.",
        :NOT_EVALUATED => "A taxon is Not Evaluated when it is has not yet been evaluated against the criteria.",
        :REGIONALLY_EXTINCT => "Category for a taxon when there is no reasonable doubt that the last individual potentially capable of reproduction within the region has died or has disappeared from the wild in the region, or when, if it is a former visiting taxon, the last individual has died or disappeared in the wild from the region.",
        :VULNERABLE => "A taxon is Vulnerable when the best available evidence indicates that it meets any of the criteria A to E for Vulnerable (see Section V), and it is therefore considered to be facing a high risk of extinction in the wild.",
   ),
   taxonomicStatus = Dict(
        :ACCEPTED => "",
        :DOUBTFUL => "Treated as accepted, but doubtful whether this is correct.",
        :HETEROTYPIC_SYNONYM => "More specific subclass of SYNONYM.",
        :HOMOTYPIC_SYNONYM => "More specific subclass of SYNONYM.",
        :MISAPPLIED => "More specific subclass of SYNONYM.",
        :PROPARTE_SYNONYM => "More specific subclass of SYNONYM.",
        :SYNONYM => "A general synonym, the exact type is unknown.",
   ),
)

"""
    enum()
    enum(k::Symbol)

Get the enum values for the keyword `k`, or for all enums in a `NamedTuple`.

Enum keywords are in $(keys(ENUMS)).
"""
enum(k::Symbol) = ENUMS[k]
enum() = ENUMS
